@@ -33,7 +33,7 @@
  * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
  * THE POSSIBILITY OF SUCH DAMAGES.
  *
- * $Id: compat.h 3041 2007-12-13 02:18:16Z proski $
+ * $Id: compat.h 3310 2008-01-30 20:23:49Z mentor $
  */
 #ifndef _ATH_COMPAT_H_
 #define _ATH_COMPAT_H_
@@ -42,7 +42,9 @@
 #ifdef __KERNEL__
 #include <linux/types.h>
 #include <linux/time.h>
+#include <linux/netdevice.h>
 #endif
+
 #if !defined(__KERNEL__) || !defined (__bitwise)
 #define __le16 u_int16_t
 #define __le32 u_int32_t
@@ -54,11 +56,23 @@
 #endif
 
 #ifndef container_of
-#define container_of(ptr, type, member) ({			\
+#define container_of(ptr, type, member) ({				\
 	    const typeof( ((type *)0)->member ) *__mptr = (ptr);	\
 	    (type *)( (char *)__mptr - offsetof(type,member) );})
 #endif
 
+#ifndef list_for_each_entry_reverse
+#define list_for_each_entry_reverse(pos, head, member)			\
+	for (pos = list_entry((head)->prev, typeof(*pos), member);	\
+	     prefetch(pos->member.prev), &pos->member != (head); 	\
+	     pos = list_entry(pos->member.prev, typeof(*pos), member))
+#endif
+
+#ifndef NETDEV_TX_OK
+#define NETDEV_TX_OK    0
+#define NETDEV_TX_BUSY  1
+#endif
+
 /*
  * BSD/Linux compatibility shims.  These are used mainly to
  * minimize differences when importing necesary BSD code.
