@@ -29,7 +29,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * $Id: ieee80211_ioctl.h 3244 2008-01-25 00:29:30Z mtaylor $
+ * $Id: ieee80211_ioctl.h 3367 2008-03-01 07:59:04Z mentor $
  */
 #ifndef _NET80211_IEEE80211_IOCTL_H_
 #define _NET80211_IEEE80211_IOCTL_H_
@@ -51,13 +51,14 @@ struct ieee80211_nodestats {
 	u_int32_t ns_rx_ucast;		/* rx unicast frames */
 	u_int32_t ns_rx_mcast;		/* rx multi/broadcast frames */
 	u_int64_t ns_rx_bytes;		/* rx data count (bytes) */
-	u_int64_t ns_rx_beacons;		/* rx beacon frames */
+	u_int64_t ns_rx_beacons;	/* rx beacon frames */
 	u_int32_t ns_rx_proberesp;	/* rx probe response frames */
 
 	u_int32_t ns_rx_dup;		/* rx discard because it's a dup */
 	u_int32_t ns_rx_noprivacy;	/* rx w/ wep but privacy off */
-	u_int32_t ns_rx_wepfail;		/* rx wep processing failed */
+	u_int32_t ns_rx_wepfail;	/* rx wep processing failed */
 	u_int32_t ns_rx_demicfail;	/* rx demic failed */
+	u_int32_t ns_rx_hwdemicerr;	/* RX HW demic spurious failure */
 	u_int32_t ns_rx_decap;		/* rx decapsulation failed */
 	u_int32_t ns_rx_defrag;		/* rx defragmentation failed */
 	u_int32_t ns_rx_disassoc;	/* rx disassociation */
@@ -79,7 +80,7 @@ struct ieee80211_nodestats {
 
 	u_int32_t ns_tx_eosplost;	/* uapsd EOSP retried out */
 
-	u_int32_t ns_ps_discard;		/* ps discard due to of age */
+	u_int32_t ns_ps_discard;	/* ps discard due to of age */
 
 	u_int32_t ns_uapsd_triggers;	/* uapsd triggers */
 
@@ -632,12 +633,15 @@ enum {
 	IEEE80211_PARAM_TXCONT_RATE		= 68,	/* continuous transmit mode data rate (in mbit/sec) - will use closest match from current rate table */
 	IEEE80211_PARAM_TXCONT_POWER		= 69,	/* power level in units of 0.5dBm */
 	IEEE80211_PARAM_DFS_TESTMODE		= 70,	/* do not perform DFS actions (i.e. markng DFS and channel change on interference), just report them via debug. */
-	IEEE80211_PARAM_DFS_CHANCHECKTIME	= 71,	/* how long do we wait for chan availability
+	IEEE80211_PARAM_DFS_CACTIME		= 71,	/* how long do we wait for chan availability
 							   scans ?
 							   FCC requires 60s, so that is the default. */
-	IEEE80211_PARAM_DFS_NONOCCUPANCYPERIOD	= 72,	/* DFS no-occupancy limit - how long do we stay
+	IEEE80211_PARAM_DFS_EXCLPERIOD		= 72,	/* DFS no-occupancy limit - how long do we stay
 							   off a channel once radar is detected?
 							   FCC requires 30m, so that is the default. */
+	IEEE80211_PARAM_BEACON_MISS_THRESH	= 73,	/* Beacon miss threshold (in beacons) */
+	IEEE80211_PARAM_BEACON_MISS_THRESH_MS	= 74,	/* Beacon miss threshold (in ms) */
+	IEEE80211_PARAM_RSSI_EWMA               = 75,
 };
 
 #define	SIOCG80211STATS			(SIOCDEVPRIVATE+2)
